"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
Object.defineProperty(exports, "__esModule", { value: true });
var core_1 = require("@angular/core");
var forms_1 = require("@angular/forms");
var CONTROL_VALUE_ACCESSOR = {
    name: 'formatterParserValueAccessor',
    provide: forms_1.NG_VALUE_ACCESSOR,
    useExisting: core_1.forwardRef(function () { return FormatterParserDirective; }),
    multi: true
};
var FormatterParserDirective = (function () {
    function FormatterParserDirective(_elementRef, fps, 
        // BIG THX to https://github.com/SanderElias for this hint
        fcd) {
        this._elementRef = _elementRef;
        this.fps = fps;
        this.fcd = fcd;
        this.formatterParserView = [];
        this.formatterParserModel = [];
    }
    FormatterParserDirective.prototype.registerOnTouched = function (fn) {
        this.onTouch = fn;
    };
    FormatterParserDirective.prototype.registerOnChange = function (fn) {
        this.onModelChange = fn;
    };
    FormatterParserDirective.prototype.ngOnInit = function () {
        this.formControl = this.fcd.form.controls[this.formControlName];
        this.inputElement = this.getInputElementRef();
        this.updateFormatterAndParser();
    };
    // Parser: View to Model
    FormatterParserDirective.prototype.onControlInput = function ($event) {
        var rawValue = this.inputElement.value;
        // If there is a reactive FormControl present trigger onTouch
        if (this.onTouch) {
            this.onTouch();
        }
        // write value to view (visible text of the form control)
        this.inputElement.value = this.formatterParserView
            .reduce(function (state, transform) { return transform(state).result; }, rawValue || null);
        // write value to model (value stored in FormControl)
        var modelValue = this.formatterParserModel
            .reduce(function (state, transform) { return transform(state).result; }, rawValue || null);
        // If there is a reactive formControl present update its model
        if (this.onModelChange) {
            this.onModelChange(modelValue);
        }
    };
    // Formatter: Model to View
    FormatterParserDirective.prototype.writeValue = function (rawValue) {
        // write value to view (visible text of the form control)
        this.inputElement.value = this.formatterParserView
            .reduce(function (state, transform) { return transform(state).result; }, rawValue);
        // write value to model (value stored in FormControl)
        var modelValue = this.formatterParserModel
            .reduce(function (state, transform) { return transform(state).result; }, rawValue);
        // prevent cyclic function calls
        if (rawValue !== modelValue) {
            // If there is a reactive FormControl present update its model
            if (this.onModelChange) {
                // @TODO consider other way to call patchValue
                this.formControl.patchValue(modelValue);
            }
        }
    };
    // fetch formatter and parser form config and update props
    FormatterParserDirective.prototype.updateFormatterAndParser = function () {
        var _this = this;
        this.formatterParserView = [];
        this.formatterParserModel = [];
        if (!this.config) {
            return;
        }
        if ('formatterParser' in this.config) {
            // setup formatterParser functions for view and model values
            this.config.formatterParser
                .forEach(function (formatterConfig) {
                var targetBoth = 2;
                var fPF = _this.fps.getFormatParseFunction(formatterConfig.name, formatterConfig.params);
                var t = (formatterConfig.target === undefined) ? targetBoth : formatterConfig.target;
                if (t === 1 || t === 2) {
                    _this.formatterParserModel.push(fPF);
                }
                if ((t === 0 || t === 2)) {
                    _this.formatterParserView.push(fPF);
                }
            });
        }
    };
    // get a safe ref to the input element
    FormatterParserDirective.prototype.getInputElementRef = function () {
        var input;
        if (this._elementRef.nativeElement.tagName === 'INPUT') {
            // `textMask` directive is used directly on an input element
            input = this._elementRef.nativeElement;
        }
        else {
            // `textMask` directive is used on an abstracted input element, `ion-input`, `md-input`, etc
            input = this._elementRef.nativeElement.getElementsByTagName('INPUT')[0];
        }
        if (!input) {
            throw new Error('You can applied the "formatterParser" directive only on inputs or elements containing inputs');
        }
        return input;
    };
    return FormatterParserDirective;
}());
__decorate([
    core_1.Input('formatterParser')
], FormatterParserDirective.prototype, "config", void 0);
__decorate([
    core_1.Input()
], FormatterParserDirective.prototype, "formControlName", void 0);
__decorate([
    core_1.HostListener('input', ['$event'])
], FormatterParserDirective.prototype, "onControlInput", null);
FormatterParserDirective = __decorate([
    core_1.Directive({
        selector: '[formatterParser]',
        providers: [
            CONTROL_VALUE_ACCESSOR
        ]
    }),
    __param(2, core_1.Optional()), __param(2, core_1.Host()), __param(2, core_1.SkipSelf())
], FormatterParserDirective);
exports.FormatterParserDirective = FormatterParserDirective;
