"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var testing_1 = require("@angular/core/testing");
var formatter_parser_service_1 = require("./formatter-parser.service");
var formatter_parser_injectionToken_1 = require("./formatter-parser.injectionToken");
function customFormatterFunction(v) {
    var result = {
        result: v + '_',
        name: 'customFormatterFunction',
    };
    return result;
}
describe('FormatterParserService', function () {
    beforeEach(function () {
        testing_1.TestBed.configureTestingModule({
            providers: [
                formatter_parser_service_1.FormatterParserService,
                { provide: formatter_parser_injectionToken_1.FORMATTER_PARSER, useValue: customFormatterFunction, multi: true }
            ],
        });
    });
    it('should be created', testing_1.inject([formatter_parser_service_1.FormatterParserService], function (service) {
        expect(service).toBeTruthy();
    }));
    it('should throw if we request a not existing function', testing_1.inject([formatter_parser_service_1.FormatterParserService], function (service) {
        var functionName = 'notExistingFunction';
        expect(function () {
            service.getFormatParseFunction(functionName);
        }).toThrow(new Error("Formatter or Parser with name " + functionName + " \n                             is not provided as a function via FormatterParser \n                             service or FORMATTER_PARSER InjectionToken. \n                             Did you forgot to provide them?"));
    }));
    it('should be able to get the built in functions', testing_1.inject([formatter_parser_service_1.FormatterParserService], function (service) {
        var buildInFuncNames = ['toUpperCase', 'toLowerCase', 'replaceString', 'toCapitalized'];
        for (var i = 0; i < buildInFuncNames.length; i++) {
            var func = service.getFormatParseFunction(buildInFuncNames[i]);
            expect(typeof func).toBe('function');
        }
    }));
    it('should be able to get the built in functions', testing_1.inject([formatter_parser_service_1.FormatterParserService], function (service) {
        var buildInFuncNames = ['toUpperCase', 'toLowerCase', 'replaceString', 'toCapitalized'];
        for (var i = 0; i < buildInFuncNames.length; i++) {
            var func = service.getFormatParseFunction(buildInFuncNames[i]);
            expect(typeof func).toBe('function');
        }
    }));
    it('should throw if we request a not existing function', testing_1.inject([formatter_parser_service_1.FormatterParserService], function (service) {
        var customFormatterFunction = service.getFormatParseFunction('customFormatterFunction');
        expect(typeof customFormatterFunction).toBe('function');
        expect(customFormatterFunction('').result).toBe('_');
    }));
});
